











/*----------------------------------------------------------------------- 
 * 
 * File Name: I8CreateVector.c
 * 
 * Author: Finn, L. S.
 * 
 * Revision: $Id: CreateVector.m4,v 1.1 2000/02/16 01:58:26 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * NAME 
 * I8CreateVector
 * 
 * SYNOPSIS 
 * void I8CreateVector (Status *, INT8Vector **vector, INT4 length);
 * 
 * DESCRIPTION 
 * Create a INT8Vector object. 
 * 
 * DIAGNOSTICS 
 * Illegal length, vector == NULL, *vector != NULL, malloc failure
 *
 * CALLS
 * LALMalloc
 * 
 * NOTES
 * 
 *-----------------------------------------------------------------------
 */

#ifndef _LALSTDLIB_H
#include "LALStdlib.h"
#ifndef _LALSTDLIB_H
#define _LALSTDLIB_H
#endif
#endif

#ifndef _AVFACTORIES_H
#include "AVFactories.h"
#ifndef _AVFACTORIES_H
#define _AVFACTORIES_H
#endif
#endif

NRCSID (CREATEVECTORC, "$Id: CreateVector.m4,v 1.1 2000/02/16 01:58:26 jolien Exp $");

typedef INT8 dtype;		/* change for different factory */
typedef INT8Vector vtype;	/* change for different factory */

void I8CreateVector (Status* status, 
	    vtype** vector,
	    INT4 length) 
{
  /* 
   * Initialize status structure
   */

  INITSTATUS (status, CREATEVECTORC);	
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (length > 0, status, CREATEVECTOR_ELENGTH, CREATEVECTOR_MSGELENGTH);

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vector != NULL, status, CREATEVECTOR_EVPTR, CREATEVECTOR_MSGEVPTR);

  ASSERT (*vector == NULL, status, CREATEVECTOR_EUPTR, CREATEVECTOR_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vector = (vtype *) LALMalloc(sizeof(vtype));
  ASSERT (*vector != NULL, status,
          CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC);

  (*vector)->length = 0;	/* length 0 until storage allocated */
  (*vector)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   * Test that storage is properly allocated. Can't handle with ASSERT
   * since we need to de-allocate structure pointer before an error return
   */

  (*vector)->data = (dtype *) LALMalloc(sizeof(dtype)*length);

  if ( NULL == (*vector)->data )
  {
    /* Must free storage pointed to by *vector */
    LALFree((void *) *vector);
    ABORT (status, CREATEVECTOR_EMALLOC, CREATEVECTOR_MSGEMALLOC);
    return;
  }
  (*vector)->length = length;	/* Set length if storage allocated */

  /* We be done: Normal exit */

  RETURN (status);
}
